%define openssl_prefix          /usr/local/openresty/openssl111
%define zlib_prefix             /usr/local/openresty/zlib
%define openssl_prefix_asan     /usr/local/openresty-asan/openssl111
%define zlib_prefix_asan        /usr/local/openresty-asan/zlib
%define openssl_prefix_debug    /usr/local/openresty-debug/openssl111
%define zlib_prefix_debug       /usr/local/openresty/zlib

Summary:            OpenSSL library for OpenResty
Name:               openresty-openssl111
Version:            1.1.1w
Release:            1%{?dist}
License:            OpenSSL
URL:                https://openresty.org/
Source0:            https://www.openssl.org/source/openssl-%{version}.tar.gz

Patch0001:          openssl-1.1.1f-sess_set_get_cb_yield.patch
Patch0002:          copy-dir.patch


BuildRequires:      gcc, make, perl
BuildRequires:      openresty-zlib-devel >= 1.2.11
Requires:           openresty-zlib >= 1.2.11


%description
This OpenSSL library build is specifically for OpenResty uses. It may contain
custom patches from OpenResty.


%package devel
Summary:            Development files for OpenResty's OpenSSL library
Requires:           %{name} = %{version}-%{release}

%description devel
Provides C header and static library for OpenResty's OpenSSL library.

%package asan
Summary:            Clang AddressSanitizer Debug version of the OpenSSL library for OpenResty
BuildRequires:      ccache, gcc, make, perl, clang, compiler-rt, libasan
BuildRequires:      openresty-zlib-asan-devel >= 1.2.11-6
Requires:           openresty-zlib-asan >= 1.2.11-6

%description asan
This is the clang AddressSanitizer version of the OpenSSL library build for OpenResty uses.

%package asan-devel
Summary:            Clang AddressSanitizer version of development files for OpenResty's OpenSSL library
Requires:           openresty-openssl111-asan = %{version}-%{release}

%description asan-devel
Provides C header and static library for the clang AddressSanitizer version of OpenResty's OpenSSL library. This is the clang AddressSanitizer version.

%package debug 
Summary:            Debug version of the OpenSSL library for OpenResty
Requires:           openresty-zlib >= 1.2.11

%description debug
This is the debug version of the OpenSSL library build for OpenResty uses.

%package debug-devel
Summary:            Debug version of development files for OpenResty's OpenSSL library
Requires:           openresty-openssl111-debug = %{version}-%{release}

%description debug-devel
Provides C header and static library for the debug version of OpenResty's OpenSSL library. This is the debug version.

%prep
%autosetup -p1 -n openssl-%{version}



%build
bash ./copy-dir.sh
./config \
    shared zlib -g3 \
    enable-camellia enable-seed enable-rfc3779 \
    enable-cms enable-md2 enable-rc5 \
    enable-weak-ssl-ciphers \
    enable-ssl3 enable-ssl3-method \
    --prefix=%{openssl_prefix} \
    --libdir=lib \
    -I%{zlib_prefix}/include \
    -L%{zlib_prefix}/lib \
    -Wl,-rpath,%{zlib_prefix}/lib:%{openssl_prefix}/lib

ncpus=`nproc`
if [ "$ncpus" -gt 16 ]; then
    ncpus=16
fi
make CC='ccache gcc -fdiagnostics-color=always' -j$ncpus

cd asan
export ASAN_OPTIONS=detect_leaks=0

./config \
    no-asm \
    enable-camellia enable-seed enable-rfc3779 \
    enable-cms enable-md2 enable-rc5 \
    enable-weak-ssl-ciphers \
    enable-ssl3 enable-ssl3-method \
    shared zlib -g3 -O1 -DPURIFY \
    --prefix=%{openssl_prefix_asan} \
    --libdir=lib \
    -I%{zlib_prefix_asan}/include \
    -L%{zlib_prefix_asan}/lib \
    -Wl,-rpath,%{zlib_prefix_asan}/lib:%{openssl_prefix_asan}/lib

ncpus=`nproc`
if [ "$ncpus" -gt 16 ]; then
    ncpus=16
fi
make -j$ncpus \
    LD_LIBRARY_PATH= \
    CC="ccache gcc -fsanitize=address" \
    > /dev/stderr
cd -

cd debug
./config \
    no-asm \
    enable-camellia enable-seed enable-rfc3779 \
    enable-cms enable-md2 enable-rc5 \
    enable-weak-ssl-ciphers \
    enable-ssl3 enable-ssl3-method \
    shared zlib -g3 -O0 -DPURIFY \
    --prefix=%{openssl_prefix_debug} \
    --libdir=lib \
    -I%{zlib_prefix_debug}/include \
    -L%{zlib_prefix_debug}/lib \
    -Wl,-rpath,%{zlib_prefix_debug}/lib:%{openssl_prefix_debug}/lib

sed -i 's/ -O3 / -O0 /g' Makefile

make CC='ccache gcc -fdiagnostics-color=always' -j`nproc`
cd -


%install
make install_sw DESTDIR=%{buildroot}

chmod 0755 %{buildroot}%{openssl_prefix}/lib/*.so*
chmod 0755 %{buildroot}%{openssl_prefix}/lib/*/*.so*

rm -rf %{buildroot}%{openssl_prefix}/bin/c_rehash
rm -rf %{buildroot}%{openssl_prefix}/lib/pkgconfig
rm -rf %{buildroot}%{openssl_prefix}/misc

cd asan 
make install_sw DESTDIR=%{buildroot}

chmod +w %{buildroot}%{openssl_prefix_asan}/lib/*.so
chmod +w %{buildroot}%{openssl_prefix_asan}/lib/*/*.so

rm -rf %{buildroot}%{openssl_prefix_asan}/bin/c_rehash
rm -rf %{buildroot}%{openssl_prefix_asan}/lib/pkgconfig
rm -rf %{buildroot}%{openssl_prefix_asan}/misc
cd -

cd debug 
make install_sw DESTDIR=%{buildroot}

chmod +w %{buildroot}%{openssl_prefix_debug}/lib/*.so
chmod +w %{buildroot}%{openssl_prefix_debug}/lib/*/*.so

rm -rf %{buildroot}%{openssl_prefix_debug}/bin/c_rehash
rm -rf %{buildroot}%{openssl_prefix_debug}/lib/pkgconfig
rm -rf %{buildroot}%{openssl_prefix_debug}/misc
cd -

export QA_RPATHS=$[ 0x0002 ]

%files
%defattr(-,root,root,-)

%dir %{openssl_prefix}
%dir %{openssl_prefix}/bin
%dir %{openssl_prefix}/lib
%attr(0755,root,root) %{openssl_prefix}/bin/openssl
%attr(0755,root,root) %{openssl_prefix}/lib/*.so*
%attr(0755,root,root) %{openssl_prefix}/lib/*/*.so*


%files devel
%defattr(-,root,root,-)

%dir %{openssl_prefix}/include
%{openssl_prefix}/include/*
%{openssl_prefix}/lib/*.a

%files asan
%defattr(-,root,root,-)

%attr(0755,root,root) %{openssl_prefix_asan}/bin/openssl
%attr(0755,root,root) %{openssl_prefix_asan}/lib/*.so*
%attr(0755,root,root) %{openssl_prefix_asan}/lib/*/*.so*


%files asan-devel
%defattr(-,root,root,-)

%{openssl_prefix_asan}/include/*
%attr(0755,root,root) %{openssl_prefix_asan}/lib/*.a

%files debug
%defattr(-,root,root,-)

%attr(0755,root,root) %{openssl_prefix_debug}/bin/openssl
%attr(0755,root,root) %{openssl_prefix_debug}/lib/*.so*
%attr(0755,root,root) %{openssl_prefix_debug}/lib/*/*.so*


%files debug-devel
%defattr(-,root,root,-)

%{openssl_prefix_debug}/include/*
%attr(0755,root,root) %{openssl_prefix_debug}/lib/*.a

%changelog
* Fri Dec 8 2023 Feng Weiyao <wynnfeng@tencent.com> - 1.1.1w-1
- initial build